#include <stdio.h>
#include <termios.h>
#include <unistd.h>
#include <stdbool.h>
#include <term.h>
#include <curses.h>

#define TICK 100
#define ENABLE true
#define DISABLE false

struct data_storage
{
	int setTime; // 설정한 시간
	double setTemp; // 설정한 온도
	double currTemp; // 현재 온도
	int cookMode; // 조리 모드 (0~5)까지
	bool isTime;  // 시간(TRUE) / 온도(FALSE)모드
	bool s; // 시작(TRUE) / 정지(FALSE)
}Data_Storage = {0, 20.0, 20.0, 0, true, false};

// 콘솔 입력을 위한 라이브러리들의 선언
int ch_read;
struct termios oldterm, new_term;
void set_stdin(void);
void reset_stdin(void);
int kgetch();

// Controller와 세부 Data Process들의 함수 정의
void BackLight_Controller(bool s, bool door_info); // 3.1.1
////////////////////////////////////////////
void Display_Controller(); // 3.2.1
void Time_Temp_Display(); //3.2.2
void Mode_Display(int cookMode); // 3.2.3
////////////////////////////////////////////
void System_Controller(bool door_info, char button); // 3.3.1
void Increase10seconds_10c(); // 3.3.2
void Increase30seconds_20c(); // 3,3,3
void Time_Temp_Change(); // 3.3.5
void Mode_Change(); // Cooking Mode Select 3.3.4
void Start_End(bool enable); //  3.3.6
void Time_Counting(bool enable); // 3.3.7
void Temp_Counting(bool enable); // 3.3.8
void Beep_Sound(); // 3.3.9

void main()
{
	char button = '\t'; // 입력된 버튼의 값을 저장하는 변수. Button Data에 해당.
	bool door_info = false; // 문의 개폐 상태를 저장하는 변수. 열림/닫힘(TRUE/FALSE) 

	set_stdin(); // 콘솔 입력을 위한 함수

	while(button != 'q')
	{
		// 문을 열고 닫는 입력을 처리하는 부분
		if((button = kgetch()) != -1)
		{
			if( (button == 'f') && (Data_Storage.s == false) )
				door_info = !door_info;
		}

		BackLight_Controller(Data_Storage.s, door_info);
		Display_Controller();
		System_Controller(door_info, button);
		printf("문 상태 = %s\n", door_info ? "열림":"닫힘");

		usleep(TICK * 1000); // TICK (0.1초) 만큼 딜레이를 주는 함수
		system("clear");
	}

	reset_stdin(); // 콘솔 입력을 위한 함수
}

void BackLight_Controller(bool s, bool door_info) // 3.1.1
{
	static int state = 1; // 1은 'BackLight off' state, 2는 'BackLight On' state
	static bool enable = false;

	switch(state)
	{
	case 1:
		if( door_info == true || ((door_info == false) && (s == TRUE)) )
		{
			enable = true;
			state = 2;
		}
		break;
	case 2:
		if( (door_info == false) && (s == FALSE) )
		{
			enable = false;
			state = 1;
		}
		break;
	}

	// BackLight 3.1.2에 해당하는 부분
	if(enable == true)
		printf("\033[93m%s\n", "Display On");
	else
		printf("\033[0m%s\n", "Display Off");
}

void Display_Controller() // 3.2.1
{
	static int state = 1; // 1밖에 없음. 1은 Display Output
	
	Time_Temp_Display(Data_Storage);
	Mode_Display(Data_Storage.cookMode);
}
void Time_Temp_Display() // 3.2.2
{
	// 초를 분과 초로 나뉘어 출력해줍니다
	int min;
	if(Data_Storage.isTime == true)
	{
		min = (int) Data_Storage.setTime / 60;
		printf("%.2d : %.2d\n", min, Data_Storage.setTime - (min * 60) ); 
	}
	else
	{
		printf("%.2d / %.2dºC\n", (int) Data_Storage.currTemp, (int) Data_Storage.setTemp);
	}
}
void Mode_Display(int cookMode) // 3.2.3
{
	static char mode_string[6][15] = {"manual", "떡", "죽", "밥", "국/찌개", "피자"};
	printf("%.2d : %s\n", cookMode, mode_string[cookMode]);
}

void System_Controller(bool door_info, char button) // 3.3.1
{
	static int state = 1; // 1은 'Stop' state
	static bool TimeCount = false; // Time_Counting 3.3.7의 Enable/Disable 상태를 저장
	static bool TempCount = false; // Temp_Counting 3.3.8의 Enable/Disable 상태를 저장

	switch(state)
	{
	case 1:
		if((button == 'a') && (Data_Storage.cookMode == 0))
		{
			state = 2;	//2는 'Increase 10seconds/10c' state
			Increase10seconds_10c();
		}
		else if((button == 'b') && (Data_Storage.cookMode == 0))
		{
			state = 3; //3은 'Increase 30seconds/20c' state
			Increase30seconds_20c();
		}		
		else if(button == 'c')
		{
			state = 4;	//4는 'Time / Temp Change' state
			Time_Temp_Change();
		}
		else if((button == 'd') && Data_Storage.isTime)
		{
			state = 5;	//5는 'Mode Change' state
			Mode_Change();
		}
		else if( (button == 'e') && !door_info)
		{
			state = 6;	//6은 'Time or Temp' state
			Start_End(ENABLE); // true는 Enable, false은 Disable.
		}
		break;
	case 2:
		state = 1; // 'Stop' state로 돌아감
		break;
	case 3:
		state = 1; // 'Stop' state로 돌아감
		break;
	case 4:
		state = 1; // 'Stop' state로 돌아감
		break;
	case 5:
		state = 1; // 'Stop' state로 돌아감
		break;
	case 6:
		if(Data_Storage.isTime) // Time 이면
		{
			state = 7; // 7은 'Operation by Time' state
			TimeCount = ENABLE;
		}
		else if(!Data_Storage.isTime) // Temp 이면
		{
			state = 8; // 8은 'Operation by Temp' state
			TempCount = ENABLE;
		}
		break;
	case 7:
		if(button == 'e')
		{
			state = 1; // 'Stop' state로 돌아감
			Start_End(DISABLE);
			TimeCount = DISABLE;
			Data_Storage.setTime = 0;
		}
		else if(Data_Storage.setTime == 0)
		{
			state = 9; // 9는 'Done' state임
			TimeCount = DISABLE;
		}
		break;
	case 8:
		if(button == 'e')
		{
			state = 1; // 'Stop' state로 돌아감
			Start_End(DISABLE);
			TempCount = DISABLE;
			Data_Storage.setTemp = 20;
			Data_Storage.currTemp = 20;
		}
		else if(Data_Storage.setTemp == Data_Storage.currTemp)
		{
			state = 9; // 9는 'Done' state임
			TempCount = DISABLE;
			Data_Storage.currTemp = 20;
		}
		break;
	case 9:
		state = 1; // 'Stop' state로 돌아감
		Beep_Sound(); // Trigger 'Beep Sound'
		Start_End(DISABLE); // Disable "Start / End"
		break;
	}
	Time_Counting(TimeCount);
	Temp_Counting(TempCount);
}
void Increase10seconds_10c() // 3.3.2
{
	if(Data_Storage.isTime == true)
	{
		Data_Storage.setTime +=  10;
		if(Data_Storage.setTime > 600)
			Data_Storage.setTime = 10;
	}
	else
	{
		Data_Storage.setTemp += 10;
		if(Data_Storage.setTemp > 90)
			Data_Storage.setTemp = 30;
	}
}
void Increase30seconds_20c() // 3.3.3
{
	if(Data_Storage.isTime == true)
	{
		if(Data_Storage.setTime == 600)
			Data_Storage.setTime = 30;
		else
		{
			Data_Storage.setTime += 30;
			if(Data_Storage.setTime > 600)
				Data_Storage.setTime = 600;
		}
	}
	else
	{
		if(Data_Storage.setTemp == 90)
			Data_Storage.setTemp = 50;
		else
		{
			Data_Storage.setTemp += 30;
			if(Data_Storage.setTemp > 90)
				Data_Storage.setTemp = 90;
		}
	}
}
void Time_Temp_Change() // 3.3.5
{
	Data_Storage.isTime = !Data_Storage.isTime;
	if(Data_Storage.isTime == false)
	{
		Data_Storage.cookMode = 0;
		Data_Storage.setTemp  = 20;
	}
}
void Mode_Change() // Cooking Mode Select 3.3.4
{
	Data_Storage.cookMode = (Data_Storage.cookMode+1)%6;
	if(Data_Storage.cookMode == 0)
		Data_Storage.setTime = 0;
	else if(Data_Storage.cookMode == 1)
		Data_Storage.setTime = 60;
	else if(Data_Storage.cookMode == 2)
		Data_Storage.setTime = 90;
	else if(Data_Storage.cookMode == 3)
		Data_Storage.setTime = 120;
	else if(Data_Storage.cookMode == 4)
		Data_Storage.setTime = 300;
	else if(Data_Storage.cookMode == 5)
		Data_Storage.setTime = 120;
}
void Start_End(bool enable) // 3.3.6
{
	Data_Storage.s = enable; // Enable/Disable에 따라 변수의 값을 true/false로 변환
}
void Time_Counting(bool enable) // 3.3.7
{
	static int tick = 0;
	if(enable == true)
	{
		tick = (tick + 1)%10;
		if(tick == 0)
			Data_Storage.setTime--;
	}
	else
		tick = 0;
}
void Temp_Counting(bool enable) // 3.3.8
{
	static int tick = 0;
	if(enable == true)
	{
		tick = (tick + 1)%30;
		if(tick == 0)
			Data_Storage.currTemp += 10;
	}
	else
		tick = 0;
}
void Beep_Sound() // 3.3.9
{
	printf("\a\a\a");
}

// 아래의 함수들은 인터넷에서 퍼온 함수들입니다.
void set_stdin(void)
{
    tcgetattr(0,&oldterm);
    new_term = oldterm;
    new_term.c_lflag &= ~(ICANON | ECHO | ISIG);  // 의미는 struct termios를 찾으면 됨.
    new_term.c_cc[VMIN] = 1;
    new_term.c_cc[VTIME] = 0;
    tcsetattr(0, TCSANOW, &new_term);
 
    ch_read = -1;
}

void reset_stdin(void)
{
     tcsetattr(0, TCSANOW, &oldterm);
}

int kgetch()
{
   char ch;
   int nread;
   int ich;
    new_term.c_cc[VMIN] = 0;
    new_term.c_cc[VTIME] = 0;
    tcsetattr(0, TCSANOW, &new_term);
    nread = read(0, &ch, 1);
    new_term.c_cc[VMIN] = 1;
    new_term.c_cc[VTIME] = 0;
    tcsetattr(0, TCSANOW, &new_term);
    if (nread >= 1) {
      ch_read = (int) ch;
      return ch_read;
    } else {
      return -1;
    }
}
